;Each background loads left to right, top to bottom.
;So columnxx_1 is the farthest left column of the screen and columnxx_16 is the
;fartherst right column of the screen.
;The first byte of each column is the top of the column, the last byte is the
;bottom of the column.

;Any byte that has bit 7 set to 1 will trigger the RLE code in dbuffer_subs.asm
;and write a sequence of metatiles to the current column.
;Bits 0-6 of that same byte are used to determine how many times to load the metatile
;into memory.  The byte following a "trigger/number of times to load" byte is the metatile number.
;So for example, the code ".db $8A,$19" would write metatile $19 (25th on the list in metatile.i)
;to the current column ten times.  ($8A = %10001010 --> [bit 7=1(activate RLE), bit0-6=0001010($0A in hex or decimal 10)])

background_index:
  .dw background_beg,background_01,background_02,background_03,background_04,background_05,background_end
attribute_index:
  .dw attrib_beg,attrib_01,attrib_02,attrib_03,attrib_04,attrib_05,attrib_end


;;;;;;;;;;;;;;;;;;

;background_beg, attrib_beg, background_end, & attrib_end serve as header and footer tables.  If you don't have data
;ready to load into the buffer when at the left edge of the first screen or right edge of the last screen, the PPU
;will start loading a bunch of junk onto the screen and eventually go nuts.  These first and last tables prevent
;that from happening.

background_beg:
  .dw columnbeg_1,columnbeg_2,columnbeg_3,columnbeg_4,columnbeg_5,columnbeg_6,columnbeg_7,columnbeg_8
  .dw columnbeg_9,columnbeg_10,columnbeg_11,columnbeg_12,columnbeg_13,columnbeg_14,columnbeg_15,columnbeg_16

columnbeg_1:
  .db $00
columnbeg_2:
  .db $00
columnbeg_3:
  .db $00
columnbeg_4:
  .db $00
columnbeg_5:
  .db $00
columnbeg_6:
  .db $00
columnbeg_7:
  .db $00
columnbeg_8:
  .db $00
columnbeg_9:
  .db $00
columnbeg_10:
  .db $00
columnbeg_11:
  .db $00
columnbeg_12:
  .db $00
columnbeg_13:
  .db $8D,$00
columnbeg_14:
  .db $8D,$00
columnbeg_15:
  .db $8D,$00
columnbeg_16:
  .db $8D,$00

attrib_beg:
  .dw a_beg_1,a_beg_2,a_beg_3,a_beg_4,a_beg_5,a_beg_6,a_beg_7,a_beg_8

a_beg_1:
  .db $00
a_beg_2:       
  .db $00
a_beg_3:
  .db $00
a_beg_4:
  .db $00
a_beg_5:
  .db $00
a_beg_6:
  .db $00
a_beg_7:
  .db $55,$00,$00,$00,$00,$00,$00,$00
a_beg_8:
  .db $55,$00,$00,$00,$00,$00,$50,$55

;;;;;;;;;;;;;;;;;;
background_01:
  .dw column01_1,column01_2,column01_3,column01_4,column01_5,column01_6,column01_7,column01_8
  .dw column01_9,column01_10,column01_11,column01_12,column01_13,column01_14,column01_15,column01_16

column01_1:
  .db $8B,$00,$01,$01
column01_2:
  .db $8A,$00,$02,$01,$01
column01_3:
  .db $85,$00,$08,$0A,$0A,$11,$00,$03,$01,$01
column01_4:
  .db $85,$00,$09,$0B,$0B,$11,$00,$04,$01,$01
column01_5:
  .db $8B,$00,$01,$01
column01_6:
  .db $00,$02,$05,$87,$00,$0C,$01,$01
column01_7:
  .db $00,$03,$06,$86,$00,$0C,$10,$01,$01
column01_8:
  .db $00,$04,$07,$85,$00,$0C,$0F,$0F,$01,$01
column01_9:
  .db $87,$00,$0C,$10,$84,$0F
column01_10:
  .db $86,$00,$0E,$10,$0F,$0F,$10,$0F,$0F
column01_11:
  .db $87,$00,$0D,$83,$0F,$01,$01
column01_12:
  .db $83,$00,$02,$05,$83,$00,$0D,$10,$0F,$01,$01
column01_13:
  .db $83,$00,$04,$07,$84,$00,$0D,$83,$0F
column01_14:
  .db $8A,$00,$0D,$0F,$0F
column01_15:
  .db $8B,$00,$01,$01
column01_16:
  .db $8B,$00,$01,$01

attrib_01:
  .dw a_column01_1,a_column01_2,a_column01_3,a_column01_4,a_column01_5,a_column01_6,a_column01_7,a_column01_8

a_column01_1:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column01_2:
  .db $55,$00,$00,$00,$00,$0F,$50,$05
a_column01_3:
  .db $55,$80,$08,$00,$00,$00,$50,$05
a_column01_4:
  .db $55,$A0,$0A,$00,$00,$00,$50,$05
a_column01_5:
  .db $55,$00,$00,$00,$00,$00,$00,$00
a_column01_6:
  .db $55,$00,$80,$08,$00,$00,$50,$05
a_column01_7:
  .db $55,$00,$20,$02,$00,$00,$00,$00
a_column01_8:
  .db $55,$00,$00,$00,$00,$00,$50,$55

;;;;;;;;;;;;;;;;;;
background_02:
  .dw column02_1,column02_2,column02_3,column02_4,column02_5,column02_6,column02_7,column02_8
  .dw column02_9,column02_10,column02_11,column02_12,column02_13,column02_14,column02_15,column02_16

; rle trigger, times to load -- metatile num

column02_1:
  .db $8B,$00,$01,$01
column02_2:
  .db $8B,$00,$01,$01
column02_3:
  .db $8B,$00,$01,$01
column02_4:
  .db $00,$00,$02,$05,$87,$00,$01,$01
column02_5:
  .db $00,$00,$03,$06,$84,$00,$13,$00,$00,$01,$01
column02_6:
  .db $00,$00,$04,$07,$00,$00,$08,$84,$0A,$01,$01
column02_7:
  .db $86,$00,$09,$84,$0B,$01,$01
column02_8:
  .db $88,$00,$08,$0A,$0A,$01,$01
column02_9:
  .db $88,$00,$09,$0B,$0B,$01,$01
column02_10:
  .db $83,$00,$11,$83,$00,$08,$83,$0A,$01,$01
column02_11:
  .db $83,$00,$11,$83,$00,$09,$83,$0B,$01,$01
column02_12:
  .db $8A,$00,$02,$01,$01
column02_13:
  .db $8A,$00,$03,$01,$01
column02_14:
  .db $8A,$00,$03,$01,$01
column02_15:
  .db $8A,$00,$04,$01,$01
column02_16:
  .db $8B,$00,$01,$01

attrib_02:
  .dw a_column02_1,a_column02_2,a_column02_3,a_column02_4,a_column02_5,a_column02_6,a_column02_7,a_column02_8

a_column02_1:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column02_2:       
  .db $55,$00,$88,$00,$00,$00,$50,$05
a_column02_3:
  .db $55,$00,$AA,$00,$00,$03,$50,$05
a_column02_4:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column02_5:
  .db $55,$00,$C0,$00,$00,$00,$50,$05
a_column02_6:
  .db $55,$00,$30,$00,$00,$00,$50,$05
a_column02_7:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column02_8:
  .db $55,$00,$00,$00,$00,$00,$50,$55


;;;;;;;;;;;;;;;;;;
background_03:
  .dw column03_1,column03_2,column03_3,column03_4,column03_5,column03_6,column03_7,column03_8
  .dw column03_9,column03_10,column03_11,column03_12,column03_13,column03_14,column03_15,column03_16

column03_1:
  .db $8B,$00,$01,$01
column03_2:
  .db $8B,$00,$01,$01
column03_3:
  .db $8A,$00,$0C,$01,$01
column03_4:
  .db $89,$00,$0C,$10,$01,$01
column03_5:
  .db $84,$00,$11,$00,$00,$13,$0E,$10,$0F,$01,$01
column03_6:
  .db $84,$00,$11,$00,$00,$13,$00,$0D,$10,$01,$01
column03_7:
  .db $84,$00,$11,$00,$00,$19,$00,$00,$0D,$01,$01
column03_8:
  .db $84,$00,$13,$00,$00,$19,$83,$00,$01,$01
column03_9:
  .db $8B,$00,$01,$01
column03_10:
  .db $00,$02,$05,$88,$00,$01,$01
column03_11:
  .db $00,$03,$06,$88,$00,$01,$01
column03_12:
  .db $00,$03,$06,$88,$00,$01,$01
column03_13:
  .db $00,$04,$07,$88,$00,$01,$01
column03_14:
  .db $8A,$00,$02,$01,$01
column03_15:
  .db $8A,$00,$04,$01,$01
column03_16:
  .db $8B,$00,$01,$01

attrib_03:
  .dw a_column03_1,a_column03_2,a_column03_3,a_column03_4,a_column03_5,a_column03_6,a_column03_7,a_column03_8

a_column03_1:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column03_2:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column03_3:
  .db $55,$00,$00,$0F,$F0,$00,$50,$05
a_column03_4:
  .db $55,$00,$00,$0F,$F0,$00,$50,$05
a_column03_5:
  .db $55,$80,$08,$00,$00,$00,$50,$05
a_column03_6:
  .db $55,$A0,$0A,$00,$00,$00,$50,$05
a_column03_7:
  .db $55,$20,$02,$00,$00,$00,$50,$05
a_column03_8:
  .db $55,$00,$00,$00,$00,$00,$50,$55


;;;;;;;;;;;;;;;;;;
background_04:
  .dw column04_1,column04_2,column04_3,column04_4,column04_5,column04_6,column04_7,column04_8
  .dw column04_9,column04_10,column04_11,column04_12,column04_13,column04_14,column04_15,column04_16

column04_1:
  .db $8B,$00,$01,$01
column04_2:
  .db $8B,$00,$01,$01
column04_3:
  .db $00,$02,$05,$84,$00,$11,$83,$00,$01,$01
column04_4:
  .db $00,$03,$06,$84,$00,$19,$83,$00,$01,$01
column04_5:
  .db $00,$04,$07,$84,$00,$19,$83,$00,$01,$01
column04_6:
  .db $87,$00,$11,$83,$00,$01,$01
column04_7:
  .db $8A,$00,$02,$01,$01
column04_8:
  .db $8A,$00,$03,$01,$01
column04_9:
  .db $00,$00,$14,$00,$00,$14,$00,$00,$14,$00,$03,$01,$01
column04_10:
  .db $00,$00,$15,$18,$18,$16,$00,$00,$15,$18,$18,$01,$01
column04_11:
  .db $00,$00,$15,$18,$18,$15,$18,$18,$15,$18,$18,$01,$01
column04_12:
  .db $00,$00,$17,$00,$00,$15,$18,$18,$15,$18,$18,$01,$01
column04_13:
  .db $85,$00,$17,$00,$00,$17,$00,$03,$01,$01
column04_14:
  .db $8A,$00,$04,$01,$01
column04_15:
  .db $84,$00,$11,$84,$12,$00,$00,$01,$01
column04_16:
  .db $00,$00,$11,$86,$12,$00,$00,$01,$01

attrib_04:
  .dw a_column04_1,a_column04_2,a_column04_3,a_column04_4,a_column04_5,a_column04_6,a_column04_7,a_column04_8

a_column04_1:
  .db $55,$AA,$AA,$AA,$AA,$AA,$5A,$05
a_column04_2:
  .db $55,$AA,$AA,$66,$F4,$AA,$5A,$05
a_column04_3:
  .db $55,$66,$AA,$85,$F8,$A8,$5A,$05
a_column04_4:
  .db $55,$55,$56,$A9,$5A,$AA,$50,$05
a_column04_5:
  .db $55,$95,$D0,$0E,$DE,$50,$54,$05
a_column04_6:
  .db $55,$AA,$70,$07,$FF,$50,$55,$05
a_column04_7:
  .db $55,$AA,$5A,$45,$9A,$98,$50,$05
a_column04_8:
  .db $55,$EE,$DE,$FF,$FF,$0F,$50,$55


;;;;;;;;;;;;;;;;;;
background_05:
  .dw column05_1,column05_2,column05_3,column05_4,column05_5,column05_6,column05_7,column05_8
  .dw column05_9,column05_10,column05_11,column05_12,column05_13,column05_14,column05_15,column05_16

column05_1:
  .db $00,$00,$11,$86,$12,$00,$00,$01,$01
column05_2:
  .db $00,$00,$11,$86,$12,$00,$00,$01,$01
column05_3:
  .db $84,$00,$11,$84,$12,$00,$00,$01,$01
column05_4:
  .db $86,$00,$11,$12,$12,$00,$00,$01,$01
column05_5:
  .db $8B,$00,$01,$01
column05_6:
  .db $8A,$00,$02,$01,$01
column05_7:
  .db $83,$00,$19,$00,$00,$14,$83,$00,$03,$01,$01
column05_8:
  .db $86,$00,$16,$83,$00,$04,$01,$01
column05_9:
  .db $83,$00,$19,$00,$00,$16,$84,$00,$01,$01
column05_10:
  .db $86,$00,$16,$84,$00,$01,$01
column05_11:
  .db $02,$05,$00,$19,$00,$00,$17,$83,$00,$0C,$01,$01
column05_12:
  .db $03,$06,$87,$00,$0C,$10,$01,$01
column05_13:
  .db $04,$07,$86,$00,$0E,$10,$0F,$01,$01
column05_14:
  .db $89,$00,$0D,$0F,$01,$01
column05_15:
  .db $8A,$00,$0D,$01,$01
column05_16:
  .db $8B,$00,$01,$01

attrib_05:
  .dw a_column05_1,a_column05_2,a_column05_3,a_column05_4,a_column05_5,a_column05_6,a_column05_7,a_column05_8

a_column05_1:
  .db $55,$33,$FF,$FF,$FF,$FF,$5F,$05
a_column05_2:       
  .db $55,$00,$00,$33,$FF,$FF,$5F,$05
a_column05_3:
  .db $55,$00,$00,$00,$00,$00,$50,$05
a_column05_4:
  .db $55,$00,$30,$00,$00,$00,$50,$05
a_column05_5:
  .db $55,$00,$30,$00,$00,$00,$50,$05
a_column05_6:
  .db $55,$AA,$30,$00,$00,$00,$50,$05
a_column05_7:
  .db $55,$22,$00,$00,$00,$00,$50,$05
a_column05_8:
  .db $55,$00,$00,$00,$00,$00,$50,$55
  

;;;;;;;;;;;;;;;;;;
background_end:
  .dw columnend_1,columnend_2,columnend_3,columnend_4,columnend_5,columnend_6


columnend_1:
  .db $8D,$00
columnend_2:
  .db $8D,$00
columnend_3:
  .db $8D,$00
columnend_4:
  .db $8D,$00
columnend_5:
  .db $8D,$00
columnend_6:
  .db $8D,$00

attrib_end:
  .dw a_columnend_1,a_columnend_2,a_columnend_3

a_columnend_1:
  .db $55,$00,$00,$00,$00,$00,$00,$00
a_columnend_2:
  .db $55,$00,$00,$00,$00,$00,$00,$00
a_columnend_3:
  .db $55,$00,$00,$00,$00,$00,$00,$00


;Below is a template table to help when loading in new backgrounds.  For example, for copying inserting the next background,
;background_04, simply copy this above, do a find/replace and change all the "0z_" to "05_", then copy your re-arranged
;.db directives into the correct places.

;;;;;;;;;;;;;;;;;;
background_0z:
  .dw column0z_1,column0z_2,column0z_3,column0z_4,column0z_5,column0z_6,column0z_7,column0z_8
  .dw column0z_9,column0z_10,column0z_11,column0z_12,column0z_13,column0z_14,column0z_15,column0z_16

column0z_1:
column0z_2:
column0z_3:
column0z_4:
column0z_5:
column0z_6:
column0z_7:
column0z_8:
column0z_9:
column0z_10:
column0z_11:
column0z_12:
column0z_13:
column0z_14:
column0z_15:
column0z_16:

attrib_0z:
  .dw a_column0z_1,a_column0z_2,a_column0z_3,a_column0z_4,a_column0z_5,a_column0z_6,a_column0z_7,a_column0z_8

a_column0z_1:
a_column0z_2:
a_column0z_3:
a_column0z_4:
a_column0z_5:
a_column0z_6:
a_column0z_7:
a_column0z_8:


init_attribute1:
  .db $55,$55,$55,$55,$55,$55,$55,$55
  .db $00,$00,$80,$A0,$00,$00,$00,$00
  .db $00,$00,$08,$0A,$00,$80,$20,$00
  .db $00,$00,$00,$00,$00,$08,$02,$00
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$0F,$00,$00,$00,$00,$00,$00
  .db $50,$50,$50,$50,$00,$50,$00,$50
  .db $05,$05,$05,$05,$00,$05,$00,$55

init_attribute2:
  .db $55,$55,$55,$55,$55,$55,$55,$55
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$88,$AA,$00,$C0,$30,$00,$00
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$00,$00,$00,$00,$00,$00,$00
  .db $00,$00,$03,$00,$00,$00,$00,$00
  .db $50,$50,$50,$50,$50,$50,$50,$50
  .db $05,$05,$05,$05,$05,$05,$05,$55

;;;;;;;;;;;;;;;;;;
status_bar:
  .db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27 
  .db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27
  .db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27 
  .db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27 
  .db $27,$27,$1C,$1D,$0A,$1D,$1E,$1C,$27,$27,$0B,$0A,$1B,$27,$27,$27 
  .db $27,$27,$1C,$1D,$0A,$1D,$1E,$1C,$27,$27,$0B,$0A,$1B,$27,$27,$27
  .db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27 
  .db $27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27,$27
