set_curr_next_prev_bg:
;set_curr_next_prev_bg will increment/decrement/maintain previous and 
;next background or previous background to be rendered depending on xScroll 
;in comparison to the buffer lines.

  LDA xScroll
  CMP rightBufferLine
  BCC .right_render_next
.right_render_after_next:
  CLC
  LDA curr_Background
  ADC #$02
  STA next_Background
  JMP .next
.right_render_next:
  CLC
  LDA curr_Background
  ADC #$01
  STA next_Background

.next:
  LDA xScroll
  CMP leftBufferLine
  BCC .left_render_previous
.left_render_current:
  LDA curr_Background
  STA prev_Background
  JMP .rts
.left_render_previous:
  SEC
  LDA curr_Background
  SBC #$01
  STA prev_Background

.rts:
  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;
set_background_addresses:

;set_background_addresses sets two pointers to be used later, backgroundAddr and attrAddr.
;These pointers will be used to determine which background/attribute table in background.i
;the buffer is loading from.

  LDA leftright
  CMP #$01
  BCC .load_previous
.load_next:
  LDA next_Background
  ASL A
  TAY
  JMP .next
.load_previous:
  LDA prev_Background
  ASL A
  TAY
.next:
  LDA background_index, y
  STA backgroundAddr
  LDA background_index+1, y
  STA backgroundAddr+1

  LDA attribute_index, y
  STA attrAddr
  LDA attribute_index+1, y
  STA attrAddr+1

  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;
column_test:
  JSR find_offscreen_column

  CMP last_col
  BEQ .no_column

  STA last_col  ;if current column is different, update the last_col variable
  LDA needdraw  ;and set the column flag in needdraw to 1
  ORA #%00000001
  STA needdraw
  JMP .rts

.no_column:
  LDA #$00      ;if current cap is the same, keep last_col unchanged
  AND #%11111110  ;and set column flag in needdraw to 0
  STA needdraw
.rts
  RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;
fill_offscreen_column:

;The columns drawn outside of the visible play area are two tiles wide by twenty-six tiles high.
;The the background tiles making them up are extracted from a table of metatiles found in
;metatile.i.

;Even though the PPU screen is 30 tiles high, the top four tiles of this demo are reserved
;for the status bar, hence we only need twenty-six tiles for each half of the off-screen column.

;The background portion of the buffer string that gets stored in bg_RAM consists of 
;fifty-five bytes.  The map of the buffer sting looks like this:

;[byte 0 = high byte][bytes 1,2 = low byte for left and right half of column]
;[bytes 3-29 = 26 bg tiles, making left half of the column]
;[bytes 29-55 = 26 more bg tiles, making up the right half of column]

;The data is calculated in fill_offscreen_column and find_offscreen_column subs while PPU is
;rendering.  Then, if the column_test sub determines a NEW column needs to be written offscreen,
;the contents mentioned above are loaded to the PPU quickly in load_offscreen_tiles sub when NMI fires.

.bg_highbyte:

  LDA initializeSet  ;if initialization is running, skip this first section
  CMP #$01           ;as high byte and low byte must be set differently
  BEQ .bg_tiles      ;while the first two screens are loading in init

  LDA leftright      ;the next fourteen lines caluclate which nametable to draw to
  CMP #$01
  BCC .left
.right:
  LDX xScroll
  CPX #$D0    ;compare to decimal 208
  BCS .this_nametable
  JMP .pull_nametableNum
.left:
  LDX xScroll
  CPX #$30    ;compare to decimal 48
  BCS .this_nametable

.pull_nametableNum:
  LDA nametableNum    ;0=$2000, 1=$2400
  EOR #%00000001      ;flip value in nametableNum, so it draws on opposite nametable
  JMP .next

.this_nametable:
  LDA nametableNum    ;0=$2000, 1=$2400

.next:                ;pull the address high byte
  CMP #$01
  BCC .store20
.store24:             ;need to draw on $2400, so high byte will be #$24 for entire column
  LDA #$24
  STA bg_RAM
  JMP .bg_lowbyte
.store20:
  LDA #$20            ;need to draw on $2000, so high byte will be #$20 for entire column
  STA bg_RAM

.bg_lowbyte:          ;two low bytes need stored, one for the topmost tile on left half of column, the other for topmost tile on right half of column

  JSR find_offscreen_column    ;pull first low address depending on xScroll
  CLC
  ADC #$80                     ;add decimal 128 so buffer starts drawing four rows from top of screen
  STA bg_RAM+1                 ;as to allow room for the status bar which is drawn in Init_PPU.asm
  CLC
  ADC #$01                     ;add 1 to first PPU low byte value, for the address of the right half of 2x28 tile column
  STA bg_RAM+2                 ;store it in bg_RAM

.bg_tiles:

  LDA initializeSet
  CMP #$01
  BCC .not_init         ;skip to regular pointer set up if initialization isn't running

  LDY xScroll
  LDA Offscreen_buffer_lo_table, y
  LSR A     ;first divide by two, because Offscreen_buffer_LO table holds values 0-31 (32 columns) but background.i tables are only columns 0-15 (16 columns)
  ASL A     ;multiply that number times two, so we load correctly from backgroundAddr pointer when storing current column into ptr1
  TAY
  JMP .next1

.not_init
  JSR find_offscreen_column   ;setting pointer to background file
  LSR A     ;just as above, divide by two then multiply by two, to make up for the column difference
  ASL A     
  TAY

.next1:
  LDA [backgroundAddr], y
  STA ptr1
  INY
  LDA [backgroundAddr], y
  STA ptr1+1            ;set to correct column

.column:
  LDY #$00      ;pointer is set, reset y
  LDX #$00
  STX loop_counter  ;reset loop_counter variable

.column_loop:
  LDA loop_counter
  CMP #$0D          ;compare to decimal 13 to test that we've looped 13 times
  BCS .rts

  LDA [ptr1], y
  AND #%10000000    ;look at bit 7 for RLE test bit
  BEQ .column_next  ;if 0, don't go to RLE, if 1, jump to RLE
  JSR RLE
  JMP .column_loop
.column_next:

  TYA
  PHA
  LDA [ptr1], y
  ASL A
  TAY

  LDA metatiles, y
  STA metatile_ptr
  LDA metatiles+1, y
  STA metatile_ptr+1     ;set to correct metatile

  LDY #$00
  LDA [metatile_ptr], y
  STA bg_RAM+3, x        ;upper left of metatile
  INY
  LDA [metatile_ptr], y
  STA bg_RAM+29, x       ;upper right of metatile
  INY
  INX
  LDA [metatile_ptr], y
  STA bg_RAM+3, x        ;lower left of metatile
  INY
  LDA [metatile_ptr], y
  STA bg_RAM+29, x       ;lower right of metatile
  INX

  INC loop_counter

  PLA
  TAY
  INY

  JMP .column_loop

.rts:
  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;
find_offscreen_column:
;setting the out of view column to render from xscroll on current screen
  LDA leftright
  CMP #$01
  BCC .left
.right:
  LDA rightBufferLine
  TAY
  JMP .rts
.left:
  LDA leftBufferLine
  TAY
.rts:
  LDA Offscreen_buffer_lo_table, y
  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;
RLE:
  TYA
  PHA                    ;push current y to stack, so we hold position on background.i when returning to main loop
  LDA [ptr1], y          
  AND #%01111111         ;AND to extract the number of times to load metatile
  PHA                    ;transfer the value to stack for later
  INY
  LDA [ptr1], y          ;load the metatile number
  ASL A                  ;multiply by two
  TAY                    ;transfer to use for metatile pointer

  LDA metatiles, y
  STA metatile_ptr
  LDA metatiles+1, y
  STA metatile_ptr+1     ;set to correct metatile

.RLE_loop:

  LDY #$00
  LDA [metatile_ptr], y
  STA bg_RAM+3, x        ;upper left of metatile
  INY
  LDA [metatile_ptr], y
  STA bg_RAM+29, x       ;upper right of metatile
  INY
  INX
  LDA [metatile_ptr], y
  STA bg_RAM+3, x        ;lower left of metatile
  INY
  LDA [metatile_ptr], y
  STA bg_RAM+29, x       ;lower right of metatile
  INX

  INC loop_counter

  PLA      ;pull number of times to load from stack
  SEC
  SBC #$01 ;decrement that value by one
  CMP #$00
  BEQ .rts
  PHA      ;push number of times to load back to stack
  JMP .RLE_loop

.rts:
  PLA      ;pull original y value from stack
  TAY
  INY
  INY      ;increment y twice, so it skips to correct byte on background.i when returning to main loop

  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;
fill_offscreen_attributes:

;fill_offscreen_attributes and find_offscreen_attr work similar to the background tile
;portion of the buffer.  However, since the first fifty-five bytes of bg_RAM have been used by
;background bytes, these two subs write to the 91st byte through 114th byte.

;What's stored on bytes 56-90?  ...NOTHING!  Attributes were stored in bytes 91-114 on a much earlier version of this demo and I just haven't changed it yet.

;filling the high address of attributes for column
  LDA leftright
  CMP #$01
  BCC .left
.right:
  LDX xScroll
  CPX #$D0    ;compare to decimal 208
  BCS .this_nametable
  JMP .pull_nametableNum
.left:
  LDX xScroll
  CPX #$30    ;compare to decimal 32
  BCS .this_nametable

.pull_nametableNum:
  LDA nametableNum    ;0=$2000, 1=$2400
  EOR #%00000001      ;flip value in nametableNum, so it draws on opposite nametable
  TAY
  JMP .next

.this_nametable:
  LDA nametableNum    ;0=$2000, 1=$2400
  TAY

.next:
  LDX #$00
.high_attr_loop:
  LDA attr_high, y      ;keep y the same, as we are simply storing the same
  STA bg_RAM+90, x      ;value ($23 or $27) eight times

  INX
  INX
  INX

  CPX #$18      ;compare to decimal 24, >8< high address bytes loaded, 1 every >3< bytes in buffer
  BCC .high_attr_loop

;fillng the low address of attributes for column
  JSR find_offscreen_attr
  TAY
  LDA Offscreen_attr_lo_table_2006addr, y
  STA bg_RAM+91

  LDX #$03      ;start x at 3 since we already saved the first byte
.lo_attr_loop:
  CLC
  ADC #$08
  STA bg_RAM+91, x

  INX
  INX
  INX

  CPX #$18     ;compare to decimal 24, >8< low address bytes loaded, 7 here and 1 above,
               ;1 every >3< bytes in buffer

  BCC .lo_attr_loop


;filling the attribute byte of attributes for column
  JSR find_offscreen_attr
  ASL A
  TAY

  LDA [attrAddr], y
  STA ptr1
  INY
  LDA [attrAddr], y
  STA ptr1+1            ;set to correct attribute column

  LDX #$00
  LDY #$00
.attr_byte_loop:
  LDA [ptr1], y
  STA bg_RAM+92, x
  INY

  INX
  INX
  INX
  CPX #$18     ;compare to decimal 24, >8< attribute bytes loaded, 1 every >3< bytes in buffer
  BCC .attr_byte_loop


  RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;
find_offscreen_attr:
  LDA leftright
  CMP #$01
  BCC .left

.right:
  LDA rightBufferLine
  TAY
  JMP .rts
.left:
  LDA leftBufferLine
  TAY

.rts:
  LDA Offscreen_attr_lo_table_pull, y

  RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;
load_offscreen_tiles:
  LDY #$00
.left:
  LDA bg_RAM, y
  STA $2006  ;write the high byte
  INY

  LDA bg_RAM, y
  STA $2006  ;write the low byte
  INY
  INY        ;increment y twice, skipping low byte for right half of 2x26 tile column

  LDA initializeSet   ;if initialization is running, the soft_2000 variable there
  CMP #$01            ;will cause havoc because it has the NMI turned off.
  BEQ .left_loop      ;must skip to loop if init is running.

  LDA soft_2000
  ORA #%00000100
  STA $2000

.left_loop
  LDA bg_RAM, y
  STA $2007  ;load the tile
  INY
  
  CPY #$1D  ;compare to decimal 29, high byte, low byte, skipped low byte, followed by twenty-six bytes for the tile numbers
  BCC .left_loop

  LDY #$00
.right:
  LDA bg_RAM, y
  STA $2006  ;write the high byte
  INY
  INY        ;increment y twice, skipping low byte for left half of 2x26 tile column

  LDA bg_RAM, y
  STA $2006  ;write the low byte
  INY

  LDA initializeSet   ;if initialization is running, the soft_2000 variable there
  CMP #$01            ;will cause havoc because it has the NMI turned off.
  BEQ .pre_right_loop ;must skip to loop if init is running.

  LDA soft_2000
  ORA #%00000100
  STA $2000

.pre_right_loop:
  LDY #$1D   ; set y to decimal 29, so it loads from correct position of bg_RAM
.right_loop
  LDA bg_RAM, y
  STA $2007  ;load the tile
  INY
  
  CPY #$37  ;compare to decimal 55, high byte, skipped low byte, low byte, skipped twenty-six low bytes, followed by twenty-six bytes for the tile numbers
  BCC .right_loop

.rts:
  RTS

load_offscreen_attributes:
  LDY #$5A  ;begin at decimal 90, as bytes 00-89 were filled by load_offscreen_tiles
.loop
  LDA bg_RAM, y
  STA $2006  ;write the high byte
  INY

  LDA bg_RAM, y
  STA $2006  ;write the low byte
  INY

  LDA bg_RAM, y
  STA $2007  ;load the attribute
  INY

  CPY #$72   ;compare to decimal 114, first we offset from decimal 90, then add decimal 24
             ;8 spots in buffer x 3 bytes(high, low, attribute byte)
  BCC .loop

  RTS
