init_ppu:

;;;;;;;;;;;;;;;;
;Initial PPU Settings after reset
;;;;;;;;;;;;;;;

  LDA #%10010000   ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1
  STA soft_2000
  LDA #%00011110   ; enable sprites, enable background, no clipping on left side
  STA soft_2001

;;;;;;;;;;;;;;;;;;;;;;;;
Init_Palettes:

LoadPalettes:
  LDA $2002      
  LDA #$3F
  STA $2006     
  LDA #$00
  STA $2006      
  LDX #$00         
LoadPalettesLoop:
  LDA palette, x   
  STA $2007             
  INX                   
  CPX #$20              
  BNE LoadPalettesLoop


;;;;;;;;;;;;;;;;;;;;;;;;
Init_Tiles:

  LDA #%00010100   ; keep NMI disabled, sprites from Pattern Table 0, background from Pattern Table 1, inc VRAM writes by 32
  STA $2000
  
  LDA #$00
  STA xScroll
  STA init_backgrounds

  LDA #$01
  STA initializeSet

.firstScreen:
  LDX #$00
  LDY #$02
  LDA background_index, y
  STA backgroundAddr
  LDA background_index+1, y
  STA backgroundAddr+1   ;set to correct background

  LDA #$20               ;load high byte for first nametable
  STA bg_RAM, x          ;store that data in the first byte of buffer
  INX

  JMP .begin

.secondScreen:
  LDX #$00
  LDY #$04
  LDA background_index, y
  STA backgroundAddr
  LDA background_index+1, y
  STA backgroundAddr+1   ;set to correct background

  LDA #$24               ;load high byte for second nametable
  STA bg_RAM, x          ;store that data in the first byte of buffer
  INX


.begin:

  LDY xScroll            ;find low byte for column
  LDA Offscreen_buffer_lo_table, y

  CLC
  ADC #$80
  STA bg_RAM, x          ;store low byte for left half of column in buffer
  INX
  CLC
  ADC #$01
  STA bg_RAM, x          ;store low byte for right half of column in buffer

  JSR fill_offscreen_column
  JSR load_offscreen_tiles

.comp_10:
  CLC
  LDA xScroll
  ADC #$10
  STA xScroll
  INC init_backgrounds
  LDA init_backgrounds
  CMP #$10
  BCS .comp_20
  JMP .firstScreen
.comp_20:
  LDA init_backgrounds
  CMP #$20
  BCS init_attributes1
  JMP .secondScreen


;;;;;;;;;;;;;;;;;;;;;;;;
init_attributes1:

  LDA #%00010000   ; keep NMI disabled, sprites from Pattern Table 0, background from Pattern Table 1
  STA $2000

  LDA $2002  
  LDA #$23
  STA $2006  
  LDA #$C0
  STA $2006
  LDX #$00
init_attr_loop1:
  LDA init_attribute1, x
  STA $2007
  INX     
  CPX #$40
  BNE init_attr_loop1

init_attributes2:
                      
  LDA $2002  
  LDA #$27
  STA $2006  
  LDA #$C0
  STA $2006  
  LDX #$00 
init_attr_loop2:
  LDA init_attribute2, x
  STA $2007
  INX     
  CPX #$40
  BNE init_attr_loop2



init_statusbar:
  LDA $2002
  LDA #$20
  STA $2006
  LDA #$00
  STA $2006

  LDY #$00
init_statusbar_loop:
  LDA status_bar, y
  STA $2007
  INY
  CPY #$80
  BCC init_statusbar_loop

status2:     ;this is simply to load in the $2400 section of the status bar
  LDA $2002  ;must have this so sprite 0 hits a non-transparent pixel once
  LDA #$24   ;player crosses from $2000 to $2400.
  STA $2006
  LDA #$00
  STA $2006
  LDY #$00
status2loop:
  LDA #$27
  STA $2007
  INY
  CPY #$80
  BCC status2loop


Sprite0:
  LDA #$1E          ;y pos = dec 30
  STA sprite_RAM
  LDA #$FF          ;tile
  STA sprite_RAM+1
  LDA #$23          ;attr = behind bg, palette 03
  STA sprite_RAM+2
  LDA #$80          ;x pos = dec 128
  STA sprite_RAM+3

  LDA #$00
  STA initializeSet

  RTS