  .inesprg 2   ; 2x 16KB PRG code
  .ineschr 1   ; 1x  8KB CHR data
  .inesmap 0   ; mapper 0 = NROM, no bank swapping
  .inesmir 1   ; background mirroring
  

;;;;;;;;;;;;;;;

;; DECLARE SOME VARIABLES HERE
  .rsset $0000  ;;start variables at ram location 0

buttons    .rs 1        ; player 1 gamepad buttons, one bit per button
sleeping   .rs 1        ; flag variable used for waiting on next NMI

xScroll          .rs 1  ; value to offset x value at $2005 VRAM address
rightBufferLine  .rs 1  ; position off-screen right where new columns are written to nametable
leftBufferLine   .rs 1  ; position off-screen left where new columns are written ot nametable

leftright        .rs 1  ; flag variable storing movement direction
nametableNum     .rs 1  ; which nametable we are on: 1=$2000, 2=$2400
curr_Background  .rs 1  ; which background the camera is currently displaying
next_Background  .rs 1  ; which background & attribute table is currently being loaded into/from the buffer out of background.i
prev_Background  .rs 1  ; which background & attrib table is loaded if screen moves lef

init_backgrounds .rs 1  ; variable used in init_PPU sub
initializeSet    .rs 1  ; variable telling dbuffer that initialization is running

soft_2000  .rs 1        ; temp variable for $2000
soft_2001  .rs 1        ; temp variable for $2001
last_frame .rs 1        ; stores value of xScroll's previous position in case of movement
last_col   .rs 1        ; stores the previous column status so NMI handler will update offscreen column if column_test shows current cap to be different

backgroundAddr .rs 2    ; pointer used for loading background tiles into RAM
metatile_ptr   .rs 2    ; pointer used for loading individual background tiles out of metatiles
attrAddr       .rs 2    ; pointer used for loading attribute bytes into RAM
ptr1           .rs 2    ; generic pointer
loop_counter   .rs 1    ; counter to keep track of looping
needdraw       .rs 1    ; multi-flag variable to telling PPU what to draw in NMI
                        ; bits 7-1 currently undefined 
                        ; bit 0 : 0 = don't draw column, 1 = draw column

;; DECLARE SOME CONSTANTS HERE

sprite_RAM     = $0200
bg_RAM         = $0400   

;----- first 8k bank of PRG-ROM
  .bank 0
  .org $8000  ;we have two 16k PRG banks now.  We will stick our sound engine in the first one, which starts at $8000.

  .include "init_PPU.asm"    
  .include "dbuffer_subs.asm"
  .include "Offscreen_buffer_tables.i"
  .include "metatiles.i"

;----- second 8k bank of PRG-ROM    
  .bank 1
  .org $A000

  .include "background.i"

;----- third 8k bank of PRG-ROM 

  .bank 2
  .org $C000

IRQ:
  RTI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;-----------------------------------------------------------------------
;               
;  NMI
;  HANDLER
;
;----------------------------------------------------------------------
NMI:
  PHA             ;save registers in case of interrupt
  TXA
  PHA
  TYA
  PHA

.ScrollingTest:
  LDA needdraw
  AND #%00000001
  BEQ .skipcolumns

.DrawColumns:
  JSR load_offscreen_tiles
  JSR load_offscreen_attributes

.skipcolumns:
  LDA #%10010000
  STA $2000

  LDA $2002
  LDA #$00
  STA $2005
  STA $2005

.SpriteDMA:
  LDA #$00
  STA $2003       ; set the low byte (00) of the RAM address
  LDA #$02
  STA $4014       ; set the high byte (02) of the RAM address, start the transfer

.Sprite0_testone:
  BIT $2002
  BVS .Sprite0_testone
.Sprite0_testtwo:
  BIT $2002
  BVC .Sprite0_testtwo

  LDA soft_2000
  STA $2000
  LDA soft_2001
  STA $2001

  LDA xScroll
  STA $2005
  LDA #$00
  STA $2005

  LDA #$00
  STA sleeping

  PLA             ; restore registers
  TAY
  PLA
  TAX
  PLA

  RTI             ; return from interrupt


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;-----------------------------------------------------------------------
;               
;  RESET!!!!!
;  RESET !!!!!
;
;----------------------------------------------------------------------
RESET:
  SEI          ; disable IRQs
  CLD          ; disable decimal mode
  LDX #$40
  STX $4017    ; disable APU frame IRQ
  LDX #$FF
  TXS          ; Set up stack
  INX          ; now X = 0
  STX $2000    ; disable NMI
  STX $2001    ; disable rendering
  STX $4010    ; disable DMC IRQs

  JSR vblankwait ;1st wait for vblank.

clrmem:
  LDA #$00
  STA $0000, x
  STA $0100, x
  STA $0300, x
  STA $0400, x
  STA $0500, x
  STA $0600, x
  STA $0700, x
  LDA #$FE
  STA $0200, x
  INX
  BNE clrmem
   
  JSR vblankwait ;2nd wait for vblank. PPU is ready Freddie!

  JSR init_ppu

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Initialize Variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;
  LDA soft_2000
  STA $2000
  LDA soft_2001
  STA $2001


  LDA #$00
  STA nametableNum

  LDA #$01
  STA leftright
  STA curr_Background

  LDA #$03
  STA xScroll

  LDA #$FF
  STA last_col

Forever:
  INC sleeping
.loop
  LDA sleeping
  BNE .loop

  LDA xScroll
  STA last_frame

  JSR ReadController1  ;;get the current button data for player 1
  JSR Left
  JSR Right

  JSR setBufferLines
  JSR TestDirection
  JSR set_curr_next_prev_bg
  JSR set_background_addresses

  JSR column_test
  LDA needdraw
  AND #%00000001
  BEQ .skipcolumns

  JSR fill_offscreen_column
  JSR fill_offscreen_attributes

.skipcolumns:

  JMP Forever     ;jump back to Forever, infinite loop, waiting for NMI

 
ReadController1:
  LDA #$01
  STA $4016
  LDA #$00
  STA $4016
  LDX #$08
ReadController1Loop:
  LDA $4016
  LSR A            ; bit0 -> Carry
  ROL buttons      ; bit0 <- Carry
  DEX
  BNE ReadController1Loop
  RTS

;;;;;;;;;;;;;;
Left:
  LDA buttons       ; player 1 - Left
  AND #%00000010  ; only look at bit 1
  BEQ ReadLeftDone   ; branch to ReadLeftDone if button is NOT pressed (0)

;this section sets a boundary at left edge of first background.
  LDA curr_Background
  CMP #$02
  BCS .next0         ;if currently on bg 2 or higher, then allow left move
  LDA xScroll
  CMP #$01
  BCS .next0         ;if currently on bg 1 and xScroll is 0, don't allow left move
  JMP ReadLeftDone

.next0
  DEC xScroll

  LDA last_frame
  CMP #$00
  BEQ .next1
  JMP ReadLeftDone
.next1:              ;if last xScroll was 00 and has now moved to FF
  LDA xScroll        ;we need to switch name tables going left
  CMP #$FF
  BNE ReadLeftDone
  DEC curr_Background
  JMP SwitchScreen

ReadLeftDone:
  RTS

Right:
  LDA buttons       ; player 1 - Right
  AND #%00000001    ; only look at bit 0
  BEQ ReadRightDone   ; branch to ReadRightDone0 if button is NOT pressed (0)

;this section sets a boundary at right edge of last background
  LDA curr_Background
  CMP #$04
  BCC .next0         ;if currently lower than bg 5, then allow right move
  LDA xScroll
  CMP #$FF
  BCC .next0         ;if currently on bg 5 and xScroll is FF, don't allow right move
  JMP ReadRightDone

.next0
  INC xScroll

  LDA last_frame
  CMP #$FF
  BEQ .next1
  JMP ReadRightDone
.next1:             ;if last xScroll was FF and has now moved to 00
  LDA xScroll       ;we need to switch name tables going right
  CMP #$00
  BNE ReadRightDone
  INC curr_Background 
  JMP SwitchScreen
  
ReadRightDone:
  RTS

;;;;;;;;;;;;;;

SwitchScreen:
  LDX nametableNum
  CPX #$01
  BCC .switchToOne ;nametableNum is 1, switch name table to $2400
                   ;if zero then...
  LDA #%10010000   ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1, name table $2000
  STA soft_2000    ;...switch it to $2000
  DEC nametableNum
  JMP .rts
.switchToOne
  LDA #%10010001   ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1, name table $2400
  STA soft_2000
  INC nametableNum
.rts
  RTS

;;;;;;;;;;;;;

TestDirection:

;TestDirection tests whether screen is scrolling left or right, and then sets leftright variable
;to 0 for left or 1 for right.  That is then used throughout buffer code to determine
;whether to draw the next off-screen column to the left or right of visible area.
  LDA xScroll
  CMP last_frame
  BCC .setleft
.setright:
  LDA #$01
  STA leftright
  JMP .rts
.setleft:
  LDA #$00
  STA leftright
.rts:
  RTS

;;;;;;;;;;;;;

setBufferLines:

;setBufferLines takes current position of xScroll and adds six columns for the right buffer line,
;so that new column is always being drawn six columns in front of right edge of visible area.  Sub 
;then sets the left buffer line six columns behind left edge of visible area.
  CLC
  LDA xScroll
  ADC #$30
  STA rightBufferLine
  SEC
  SBC #$60
  STA leftBufferLine
  RTS


;;;;;;;;;;;;;

vblankwait:
  BIT $2002
  BPL vblankwait
  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  
  .bank 3
  .org $E000

palette:
  .db $22,$29,$1A,$0F,  $22,$36,$17,$0F,  $22,$30,$21,$0F,  $22,$27,$17,$0F   ;;background palette
  .db $22,$16,$27,$18,  $22,$1A,$30,$27,  $22,$16,$30,$27,  $22,$0F,$36,$17   ;;sprite palette


  .org $FFFA     ;first of the three vectors starts here
  .dw NMI        ;when an NMI happens (once per frame if enabled) the 
                   ;processor will jump to the label NMI:
  .dw RESET      ;when the processor first turns on or is reset, it will jump
                   ;to the label RESET:
  .dw 0          ;external interrupt IRQ is not used in this tutorial
  
  
;;;;;;;;;;;;;;  
  
  
  .bank 4
  .org $0000
  .incbin "mario.chr"   ;includes 8KB graphics file from SMB1
  